%
%                              Copyright 2008 Cleimar Jos Salvador, 
%						                      Diego da Silva de Medeiros, 
%						                      Fabrcio Emydio da Silva, 
%						                      Patrick Hedlund Rosa, 
%						                      Marcos Moecke, 
%						                      Mario de Noronha Neto
%
%
%    This file is part of Edge2javaTX.
%
%    Edge2javaTX is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    Edge2javaTX is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with Edge2javaTX. If not, see <http://www.gnu.org/licenses/>.
%

function [PDU,PDUlength] = make_PDU(tlfn,IP)
%% Portas de encaminhamento da mensagem SMS
% Porta de destino (porta da unidade mvel destino)
portaD = 16005;
% Porta de origem (porta do mdulo)
portaO = 0;

%% 01: SCAL - Service Center Address Length
% Igualado  zero, indicando que o SCA ser buscado do comando AT+CSCA
SCAL = '00';

%% 02: TPDU - PDU Type
%     - RP:   (0) - caminho de retorno no ser setado
%             (1) - caminho de retorno ser setado
RP   = 0;
%     - UDHI: (0) - campo User Data (UD) conter somente uma mensagem
%             (1) - campo User Data (UD) conter um UDH
global PDU_header;
if PDU_header
	UDHI = 1;
else
	UDHI = 0;
end
%     - SRR:  (0) - relatrio de status no ser retornado
%             (1) - relatrio de status ser retornado
SRR  = 1;
%     - VPF:  (00) - campo VP no estar presente
%             (10) - campo VP estar na forma relativa
VPF1  = 0;
VPF2  = 0;
%     - RD:   (0) - SMSC aceitar mais uma mensagem
%             (1) - SMSC no aceitar mais uma mensagem
RD   = 0;
%     - MTI: indica o tipo da mensagem. 
%             (01) indica uma mensagem enviada
MTI1 = 0;
MTI2 = 1;

% Converte para Hexadecimal o resultado
TPDU = dec2hex(bi2de([RP UDHI SRR VPF1 VPF2 RD MTI1 MTI2],'left-msb'),2);

%% 03: MR - Message Reference
% Indica o nmero da mensagem
MR = '00';

%% 04: DA - Destination Address
%
%     - LEN: nmero de dgitos do telefone (TEL)
LEN = dec2hex(size(tlfn,2),2);
%     - TON: (91) indica que o nmero a seguir est em formato internacional
%            (81) indica que o nmero a seguir est em formato nacional
TON = '81';
% Invertendo o nmero de telefone
TEL = [];
for R = 1:2:size(tlfn,2)
	TEL = [TEL tlfn(R+1) tlfn(R)];
end

% Junta tudo:
DA = [LEN TON TEL];

%% 05: PID - Protocol Identifier
% (00) diz que o PDU ser tratado como um SMS
PID = '00';

%% 06: DCS - Data Coding Scheme
% (F6) indica que a mensagem ser formada por caracteres de 8 bits
% (00) indica que a mensagem ser formada por caracteres de 7 bits (GSM)
DCS = '00';

%% 07: VP - Vality Period
% indica o perodo de validade da mensagem.
% setamos no campo VPF do TPDU que a mensagem no ter VP
VP = '';

%% 08: UDL - User Data Length & UD - User Data
% utilizamos a funo make_UD para realizar a codifcao em 7 bits do corpo
% da mensagem SMS
[UDL,UD] = make_UD(IP,portaD,portaO);

%% Concatena tudo e pega o nmero de bytes
PDU = [SCAL TPDU MR DA PID DCS VP UDL UD];
% 1  subtrado porque no clculo deve ser retirado os caracteres
% indicadores do Centro de servio
PDUlength = size(PDU,2)/2 - 1;
